/*
 * Copyright (c) 1999-2019 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_RECT_HPP
#define PW_CORE_RECT_HPP

#include <pw/core/point.hpp>
#include <pw/core/size.hpp>

namespace pw {

template <typename T_>
struct rectangle_ {

	size_<T_> size;
	point_<T_> position;

	rectangle_() = default;

	rectangle_(point_<T_> const & p,size_<T_> const & s) : size(s), position(p) {}

	bool contains(const point_<T_>& p) const noexcept
    {
        return p.x >= position.x && p.x <= position.x + size.width &&
			   p.y >= position.y && p.y <= position.y + size.height;
    }

    template <typename To_>
	rectangle_<To_> cast() const { return rectangle_<To_>(position.template cast<To_>(),size.template cast<To_>()); }

};

using rectangle = rectangle_<real_t>;
using rectanglef = rectangle_<float>;
using rectangled = rectangle_<double>;

}

#endif
